/**
 *  '$RCSfile: DataFileUploadInterface.java,v $'
 *    Purpose: An abstract class that is a template for a data transfer class
 *  Copyright: 2000 Regents of the University of California and the
 *             National Center for Ecological Analysis and Synthesis
 *    Authors: Chad Berkley
 *    Release: @release@
 *
 *   '$Author: berkley $'
 *     '$Date: 2001/01/18 23:15:20 $'
 * '$Revision: 1.4 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package edu.ucsb.nceas.metacat;

import java.net.*;
import java.io.*;
import java.util.Properties;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.sql.*;

abstract class DataFileUploadInterface extends Thread 
{
  private MetaCatUtil util = new MetaCatUtil();
  private static String filedir = "";
  private String user = null;
  private String sess_id = null;
  private int port;
  protected Socket s;
  
  /**
   * Sets the port, user and sess_id
   */
  DataFileUploadInterface(int port, String user, String sess_id) 
  {
    this.sess_id = sess_id;
    this.user = user;
    this.port = port;
    filedir = util.getOption("datafilepath");
  }
  
  /**
   * Override this method with the code to handle the socket connection to 
   * the port specified in the constructor.
   */
  abstract void getFile(int port, String user, String sess_id);
  
  /**
   * This method is invoked when this class is broken off into a new thread
   */
  public void run () 
  {
    getFile(port, user, sess_id);
  }
  
  /**
   * attempts to connect a socket, returns null if it is not successful
   * returns the connected socket if it is successful.
   */
  public static Socket getSocket(String host, int port)
  {
    Socket s = null;
    try
    {
      s = new Socket(host, port);
      //we could create a socket on this port so the port is not available
      //System.out.println("socket connnected");
      return s;
    }
    catch(UnknownHostException u)
    {
      System.out.println("unknown host in DataFileUploadInterface.getSocket");
    }
    catch(IOException i)
    {
      //an ioexception is thrown if the port is not in use
      //System.out.println("socket not connected");
      return s;
    }
    return s;
  }
  
  /**
   * returns true if the port specified is not in use.  false otherwise
   */
  public static boolean portIsAvailable(int port)
  {
    Socket s;
    String host = "localhost";
    try
    {
      s = new Socket(host, port);
      //s.close();
      //we could create a socket on this port so the port is not available
      //System.out.println("socket not available");
      return false;
    }
    catch(UnknownHostException u)
    {
      System.out.println("unknow host error in " + 
                         "DataFileUploadInterface.portIsAvailable");
    }
    catch(IOException i)
    {
      //an ioexception is thrown if the port is not in use
      //System.out.println("socket available");
      return true;
    }
    return true;
  }
  
  /**
   * Updates xml_documents with the new data document information
   * @param accnum the accession number of the new data file
   * @param filename the filename of the new data file
   * @param userOwner the document's owner's username
   */
  public void updateDB(String accnum, String filename, String userOwner)
  {
    SimpleDateFormat formatter = new SimpleDateFormat ("yy-MM-dd HH:mm:ss");
    java.util.Date localtime = new java.util.Date();
    String dateString = formatter.format(localtime);
    
    String sqlDateString = "to_date('" + dateString + "', 'YY-MM-DD HH24:MI:SS')";
    
    StringBuffer sql = new StringBuffer();
    sql.append("insert into xml_documents (docid, docname, doctype, ");
    sql.append("user_owner, user_updated, server_location, rev, date_created");
    sql.append(", date_updated, public_access) values ('");
    sql.append(accnum).append("','").append(filename).append("','BIN','");
    sql.append(userOwner).append("','").append(userOwner).append("','1','");
    sql.append("1',").append(sqlDateString).append(",");
    sql.append(sqlDateString).append(",'0')");
    //System.out.println("sql: " + sql.toString());
    try
    {
      Connection conn = util.openDBConnection();
      PreparedStatement pstmt = conn.prepareStatement(sql.toString());
      pstmt.execute();
      pstmt.close();
      conn.close();
    }
    catch(Exception e)
    {
      System.out.println("error with db connection in DataFileServer.updateDB" +
                         ": " + e.getMessage());
    }
    
  }
}
